@@ -5,8 +5,9 @@ import logging
 import requests
 import datetime
 from flask import Response as FlaskResponse
-from six.moves.urllib_parse import urljoin
 from requests.models import Response
+from six.moves.urllib_parse import urljoin
+from moto.apigateway.models import apigateway_backends
 from localstack.utils import common
 from localstack.config import TEST_KINESIS_URL, TEST_SQS_URL
 from localstack.constants import APPLICATION_JSON, PATH_USER_REQUEST, TEST_AWS_ACCOUNT_ID
@@ -197,7 +198,8 @@ def update_content_length(response):
 
 
 def apply_template(integration, req_res_type, data, path_params={}, query_params={}, headers={}):
-    if integration['type'] in ['HTTP', 'AWS']:
+    integration_type = integration.get('type') or integration.get('integrationType')
+    if integration_type in ['HTTP', 'AWS']:
         # apply custom request template
         template = integration.get('%sTemplates' % req_res_type, {}).get(APPLICATION_JSON)
         if template:
@@ -270,16 +272,32 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
             return get_cors_response(headers)
         return make_error_response('Unable to find integration for path %s' % path, 404)
 
-    uri = integration.get('uri') or ''
-    integration_type = integration['type'].upper()
+    res_methods = path_map.get(relative_path, {}).get('resourceMethods', {})
+    meth_integration = res_methods.get(method, {}).get('methodIntegration', {})
+    int_responses = meth_integration.get('integrationResponses', {})
+    response_templates = int_responses.get('200', {}).get('responseTemplates', {})
 
-    if uri.startswith('arn:aws:apigateway:') and ':lambda:path' in uri:
+    return invoke_rest_api_integration(api_id, stage, integration, method, path, invocation_path, data,
+        headers, resource_path=extracted_path, context=context,
+        resource_id=resource.get('id'), response_templates=response_templates)
+
+
+def invoke_rest_api_integration(api_id, stage, integration, method, path, invocation_path, data, headers,
+        resource_path, context={}, resource_id=None, response_templates={}):
+
+    relative_path, query_string_params = extract_query_string_params(path=invocation_path)
+    integration_type = integration.get('type') or integration.get('integrationType')
+    uri = integration.get('uri') or integration.get('integrationUri')
+
+    if (uri.startswith('arn:aws:apigateway:') and ':lambda:path' in uri) or uri.startswith('arn:aws:lambda'):
         if integration_type in ['AWS', 'AWS_PROXY']:
-            func_arn = uri.split(':lambda:path')[1].split('functions/')[1].split('/invocations')[0]
+            func_arn = uri
+            if ':lambda:path' in uri:
+                func_arn = uri.split(':lambda:path')[1].split('functions/')[1].split('/invocations')[0]
             data_str = json.dumps(data) if isinstance(data, (dict, list)) else to_str(data)
 
             try:
-                path_params = extract_path_params(path=relative_path, extracted_path=extracted_path)
+                path_params = extract_path_params(path=relative_path, extracted_path=resource_path)
             except Exception:
                 path_params = {}
 
@@ -290,11 +308,13 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
             # Sample request context:
             # https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-create-api-as-simple-proxy-for-lambda.html#api-gateway-create-api-as-simple-proxy-for-lambda-test
             request_context = get_lambda_event_request_context(method, path, data, headers,
-                integration_uri=uri, resource_id=resource.get('id'))
+                integration_uri=uri, resource_id=resource_id)
+            stage_variables = get_stage_variables(api_id, stage)
 
             result = lambda_api.process_apigateway_invocation(func_arn, relative_path, data_str,
                 stage, api_id, headers, path_params=path_params, query_string_params=query_string_params,
-                method=method, resource_path=path, request_context=request_context, event_context=context)
+                method=method, resource_path=path, request_context=request_context,
+                event_context=context, stage_variables=stage_variables)
 
             if isinstance(result, FlaskResponse):
                 response = flask_to_requests_response(result)
@@ -393,10 +413,7 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
             action = uri.split(':dynamodb:action')[1].split('&Table=')[0]
 
             if 'PutItem' in action and method == 'PUT':
-                response_template = path_map.get(relative_path, {}).get('resourceMethods', {})\
-                    .get(method, {}).get('methodIntegration', {}).\
-                    get('integrationResponses', {}).get('200', {}).get('responseTemplates', {})\
-                    .get('application/json', None)
+                response_template = response_templates.get('application/json', None)
 
                 if response_template is None:
                     msg = 'Invalid response template defined in integration response.'
@@ -419,7 +436,7 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
                 response = requests_response(event_data, headers=aws_stack.mock_aws_request_headers())
                 return response
         else:
-            msg = 'API Gateway action uri "%s" not yet implemented' % uri
+            msg = 'API Gateway action uri "%s", integration type %s not yet implemented' % (uri, integration_type)
             LOGGER.warning(msg)
             return make_error_response(msg, 404)
 
@@ -432,7 +449,7 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
         if isinstance(data, dict):
             data = json.dumps(data)
 
-        result = function(integration['uri'], data=data, headers=headers)
+        result = function(uri, data=data, headers=headers)
 
         # apply custom response template
         data = apply_template(integration, 'response', data)
@@ -448,11 +465,18 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
         return get_cors_response(headers)
 
     msg = ('API Gateway integration type "%s", method "%s", URI "%s" not yet implemented' %
-           (integration['type'], method, integration.get('uri')))
+           (integration_type, method, uri))
     LOGGER.warning(msg)
     return make_error_response(msg, 404)
 
 
+def get_stage_variables(api_id, stage):
+    region_name = [name for name, region in apigateway_backends.items() if api_id in region.apis][0]
+    api_gateway_client = aws_stack.connect_to_service('apigateway', region_name=region_name)
+    response = api_gateway_client.get_stage(restApiId=api_id, stageName=stage)
+    return response.get('variables', None)
+
+
 def get_lambda_event_request_context(method, path, data, headers, integration_uri=None, resource_id=None):
     _, stage, relative_path_w_query_params = get_api_id_stage_invocation_path(path, headers)
     relative_path, query_string_params = extract_query_string_params(path=relative_path_w_query_params)
