@@ -51,7 +51,6 @@ class CloudFormationRegion(RegionBackend):
 
 
 class Stack(object):
-
     def __init__(self, metadata=None, template={}):
         self.metadata = metadata or {}
         self.template = template or {}
@@ -88,14 +87,15 @@ class Stack(object):
             self.metadata['Parameters'].append({'ParameterKey': key, 'ParameterValue': value})
 
     def describe_details(self):
-        attrs = ['StackId', 'StackName', 'Description', 'Parameters', 'StackStatusReason',
-            'StackStatus', 'Capabilities', 'Outputs', 'Tags', 'ParentId', 'RootId', 'RoleARN',
+        attrs = ['StackId', 'StackName', 'Description', 'StackStatusReason',
+            'StackStatus', 'Capabilities', 'ParentId', 'RootId', 'RoleARN',
             'CreationTime', 'DeletionTime', 'LastUpdatedTime', 'ChangeSetId']
         result = select_attributes(self.metadata, attrs)
-        for attr in ['Capabilities']:
+        result['Tags'] = self.tags
+        result['Outputs'] = self.outputs
+        result['Parameters'] = self.stack_parameters()
+        for attr in ['Capabilities', 'Tags', 'Outputs', 'Parameters']:
             result[attr] = {'member': result.get(attr, [])}
-        result['Outputs'] = {'member': self.outputs}
-        result['Parameters'] = {'member': self.stack_parameters()}
         return result
 
     def set_stack_status(self, status):
@@ -139,27 +139,40 @@ class Stack(object):
     @property
     def resources(self):
         """ Return dict of resources, parameters, conditions, and other stack metadata. """
+
         def add_params(defaults=True):
             for param in self.stack_parameters(defaults=defaults):
                 if param['ParameterKey'] not in result:
                     props = {'Value': param['ParameterValue']}
-                    result[param['ParameterKey']] = {'Type': 'Parameter',
-                        'LogicalResourceId': param['ParameterKey'], 'Properties': props}
+                    result[param['ParameterKey']] = {
+                        'Type': 'Parameter',
+                        'LogicalResourceId': param['ParameterKey'],
+                        'Properties': props
+                    }
+
         result = dict(self.template_resources)
+
         add_params(defaults=False)
+
         for name, value in self.conditions.items():
             if name not in result:
                 result[name] = {'Type': 'Parameter', 'LogicalResourceId': name, 'Properties': {'Value': value}}
         for name, value in self.mappings.items():
             if name not in result:
                 result[name] = {'Type': 'Parameter', 'LogicalResourceId': name, 'Properties': {'Value': value}}
+
         add_params(defaults=True)
+
         return result
 
     @property
     def template_resources(self):
         return self.template['Resources']
 
+    @property
+    def tags(self):
+        return aws_stack.extract_tags(self.metadata)
+
     @property
     def imports(self):
         def _collect(o, **kwargs):
@@ -194,10 +207,16 @@ class Stack(object):
         return result
 
     def stack_parameters(self, defaults=True):
-        result = {p['ParameterKey']: p for p in self.metadata['Parameters']}
+        result = {}
+        # add default template parameter values
         if defaults:
             for key, value in self.template_parameters.items():
-                result[key] = result.get(key) or {'ParameterKey': key, 'ParameterValue': value.get('Default')}
+                result[key] = {'ParameterKey': key, 'ParameterValue': value.get('Default')}
+        # add stack parameters
+        result.update({p['ParameterKey']: p for p in self.metadata['Parameters']})
+        # add parameters of change sets
+        for change_set in self.change_sets:
+            result.update({p['ParameterKey']: p for p in change_set.metadata['Parameters']})
         result = list(result.values())
         return result
 
@@ -250,12 +269,12 @@ class Stack(object):
 
 
 class StackChangeSet(Stack):
-
     def __init__(self, params={}, template={}):
         super(StackChangeSet, self).__init__(params, template)
         name = self.metadata['ChangeSetName']
         if not self.metadata.get('ChangeSetId'):
             self.metadata['ChangeSetId'] = aws_stack.cf_change_set_arn(name, change_set_id=short_uid())
+
         stack = self.stack = find_stack(self.metadata['StackName'])
         self.metadata['StackId'] = stack.stack_id
         self.metadata['Status'] = 'CREATE_PENDING'
