@@ -1,4 +1,3 @@
-from localstack.utils import persistence
 import re
 import os
 import sys
@@ -9,6 +8,7 @@ import logging
 import threading
 from requests.models import Response
 from localstack import config
+from localstack.utils import persistence
 from localstack.services import plugins
 from localstack.dashboard import infra as dashboard_infra
 from localstack.utils.aws import aws_stack
@@ -19,11 +19,11 @@ from localstack.utils.common import (
     empty_context_manager, run, is_root, TMP_THREADS, to_bytes, truncate, to_str,
     get_service_protocol, in_docker, safe_requests as requests)
 from localstack.services.infra import PROXY_LISTENERS
-from localstack.utils.aws.aws_stack import Environment, is_internal_call_context
+from localstack.utils.aws.aws_stack import Environment, is_internal_call_context, set_default_region_in_headers
 from localstack.services.generic_proxy import ProxyListener, start_proxy_server, modify_and_forward
 from localstack.services.sqs.sqs_listener import is_sqs_queue_url
 from localstack.utils.server.http2_server import HTTPErrorResponse
-from localstack.utils.aws.aws_stack import set_default_region_in_headers
+from localstack.services.cloudwatch.cloudwatch_listener import PATH_GET_RAW_METRICS
 
 LOG = logging.getLogger(__name__)
 
@@ -73,9 +73,10 @@ class ProxyListenerEdge(ProxyListener):
 
             if api in ['', None, '_unknown_']:
                 truncated = truncate(data)
-                LOG.info(('Unable to find forwarding rule for host "%s", path "%s %s", '
-                    'target header "%s", auth header "%s", data "%s"') % (
-                        host, method, path, target, auth_header, truncated))
+                if auth_header or target or data or path not in ['/', '/favicon.ico']:
+                    LOG.info(('Unable to find forwarding rule for host "%s", path "%s %s", '
+                        'target header "%s", auth header "%s", data "%s"') % (
+                            host, method, path, target, auth_header, truncated))
             else:
                 LOG.info(('Unable to determine forwarding port for API "%s" - please '
                     'make sure this API is enabled via the SERVICES configuration') % api)
@@ -91,9 +92,9 @@ class ProxyListenerEdge(ProxyListener):
         if isinstance(data, dict):
             data = json.dumps(data)
 
-        encoding_type = headers.get('content-encoding') or ''
-        if encoding_type.upper() == GZIP_ENCODING and api is not S3:
-            headers.set('content-encoding', IDENTITY_ENCODING)
+        encoding_type = headers.get('Content-Encoding') or ''
+        if encoding_type.upper() == GZIP_ENCODING.upper() and api not in [S3]:
+            headers.set('Content-Encoding', IDENTITY_ENCODING)
             data = gzip.decompress(data)
 
         lock_ctx = BOOTSTRAP_LOCK
@@ -115,7 +116,7 @@ def do_forward_request(api, method, path, data, headers, port=None):
         result = do_forward_request_inmem(api, method, path, data, headers, port=port)
     else:
         result = do_forward_request_network(port, method, path, data, headers)
-    if hasattr(result, 'status_code') and result.status_code >= 400 and method == 'OPTIONS':
+    if hasattr(result, 'status_code') and int(result.status_code) >= 400 and method == 'OPTIONS':
         # fall back to successful response for OPTIONS requests
         return 200
     return result
@@ -289,6 +290,14 @@ def get_api_from_custom_rules(method, path, data, headers):
     if b'Action=AssumeRoleWithSAML' in data_bytes or 'Action=AssumeRoleWithSAML' in path:
         return 'sts', config.PORT_STS
 
+    # CloudWatch backdoor API to retrieve raw metrics
+    if path.startswith(PATH_GET_RAW_METRICS):
+        return 'cloudwatch', config.PORT_CLOUDWATCH
+
+    # SQS queue requests
+    if ('QueueUrl=' in path and 'Action=' in path) or (b'QueueUrl=' in data_bytes and b'Action=' in data_bytes):
+        return 'sqs', config.PORT_SQS
+
     # TODO: move S3 public URLs to a separate port/endpoint, OR check ACLs here first
     stripped = path.strip('/')
     if method in ['GET', 'HEAD'] and '/' in stripped:
@@ -307,10 +316,6 @@ def get_api_from_custom_rules(method, path, data, headers):
             # assume that this is an S3 POST request with form parameters or multipart form in the body
             return 's3', config.PORT_S3
 
-    if stripped.count('/') == 1 and method == 'PUT':
-        # assume that this is an S3 PUT bucket object request with URL path `/<bucket>/object`
-        return 's3', config.PORT_S3
-
     # detect S3 requests sent from aws-cli using --no-sign-request option
     if 'aws-cli/' in headers.get('User-Agent', ''):
         return 's3', config.PORT_S3
@@ -319,9 +324,11 @@ def get_api_from_custom_rules(method, path, data, headers):
     if method == 'POST' and 'delete=' in path and b'<Delete' in data_bytes and b'<Key>' in data_bytes:
         return 's3', config.PORT_S3
 
-    # SQS queue requests
-    if ('QueueUrl=' in path and 'Action=' in path) or (b'QueueUrl=' in data_bytes and b'Action=' in data_bytes):
-        return 'sqs', config.PORT_SQS
+    # Put Object API can have multiple keys
+    if stripped.count('/') >= 1 and method == 'PUT':
+        # assume that this is an S3 PUT bucket object request with URL path `/<bucket>/object`
+        # or `/<bucket>/object/object1/+`
+        return 's3', config.PORT_S3
 
 
 def get_service_port_for_account(service, headers):
