@@ -6,7 +6,7 @@ import logging
 from localstack import config
 from localstack.constants import TEST_AWS_ACCOUNT_ID, MOTO_ACCOUNT_ID
 from localstack.utils.aws import aws_stack
-from localstack.utils.common import to_str, save_file, TMP_FILES, mkdir
+from localstack.utils.common import to_str, save_file, TMP_FILES, mkdir, replace_response_content
 from localstack.utils.tagging import TaggingService
 from localstack.services.generic_proxy import ProxyListener
 from localstack.services.events.scheduler import JobScheduler
@@ -19,11 +19,6 @@ EVENTS_TMP_DIR = os.path.join(config.TMP_FOLDER, 'cw_events')
 RULE_SCHEDULED_JOBS = {}
 
 
-def _replace(response, pattern, replacement):
-    content = to_str(response.content)
-    response._content = re.sub(pattern, replacement, content)
-
-
 def fix_account_id(response):
     return aws_stack.fix_account_id_in_arns(response, existing=MOTO_ACCOUNT_ID, replace=TEST_AWS_ACCOUNT_ID)
 
@@ -32,7 +27,7 @@ def fix_date_format(response):
     """ Normalize date to format '2019-06-13T18:10:09.1234Z' """
     pattern = r'<CreateDate>([^<]+) ([^<+]+)(\+[^<]*)?</CreateDate>'
     replacement = r'<CreateDate>\1T\2Z</CreateDate>'
-    _replace(response, pattern, replacement)
+    replace_response_content(response, pattern, replacement)
 
 
 def _create_and_register_temp_dir():
@@ -60,7 +55,8 @@ def get_scheduled_rule_func(data):
         for target in targets:
             arn = target.get('Arn')
             event = json.loads(target.get('Input') or '{}')
-            aws_stack.send_event_to_target(arn, event)
+            attr = aws_stack.get_events_target_attributes(target)
+            aws_stack.send_event_to_target(arn, event, target_attributes=attr)
     return func
 
 
@@ -145,8 +141,8 @@ class ProxyListenerEvents(ProxyListener):
             # fix dates returned from this API (fixes an issue with Terraform)
             fix_date_format(response)
 
-            # fix content-length header
-            response.headers['content-length'] = len(response._content)
+            # fix Content-Length header
+            response.headers['Content-Length'] = len(response._content)
 
 
 # instantiate listener
