@@ -38,7 +38,7 @@ SERVER_CERT_PEM_FILE = 'server.test.pem'
 USE_HTTP2_SERVER = config.USE_HTTP2_SERVER
 
 # CORS constants
-CORS_ALLOWED_HEADERS = ['authorization', 'content-type', 'content-md5', 'cache-control',
+CORS_ALLOWED_HEADERS = ['authorization', 'content-type', 'content-length', 'content-md5', 'cache-control',
     'x-amz-content-sha256', 'x-amz-date', 'x-amz-security-token', 'x-amz-user-agent',
     'x-amz-target', 'x-amz-acl', 'x-amz-version-id', 'x-localstack-target', 'x-amz-tagging']
 if EXTRA_CORS_ALLOWED_HEADERS:
@@ -108,13 +108,21 @@ class RegionBackend(object):
 
     @classmethod
     def get(cls, region=None):
+        regions = cls.regions()
+        region = region or cls.get_current_request_region()
+        regions[region] = regions.get(region) or cls()
+        return regions[region]
+
+    @classmethod
+    def regions(cls):
         if not hasattr(cls, 'REGIONS'):
             # maps region name to region backend instance
             cls.REGIONS = {}
+        return cls.REGIONS
 
-        region = region or aws_stack.get_region()
-        cls.REGIONS[region] = cls.REGIONS.get(region) or cls()
-        return cls.REGIONS[region]
+    @classmethod
+    def get_current_request_region(cls):
+        return aws_stack.get_region()
 
 
 # ---------------------
