@@ -2,7 +2,6 @@ import re
 import json
 import random
 import cbor2
-import base64
 from requests.models import Response
 from localstack import config
 from localstack.constants import APPLICATION_JSON, APPLICATION_CBOR
@@ -80,7 +79,6 @@ class ProxyListenerKinesis(ProxyListener):
         action = headers.get('X-Amz-Target')
         data = self.decode_content(data or '{}')
         response._content = self.replace_in_encoded(response.content or '')
-
         records = []
         if action in (ACTION_CREATE_STREAM, ACTION_DELETE_STREAM):
             event_type = (event_publisher.EVENT_KINESIS_CREATE_STREAM if action == ACTION_CREATE_STREAM
@@ -150,9 +148,9 @@ class ProxyListenerKinesis(ProxyListener):
 
             for record in records:
                 if sdk_v2:
-                    record['ApproximateArrivalTimestamp'] = int(record['ApproximateArrivalTimestamp'] * 1000)
+                    record['ApproximateArrivalTimestamp'] = int(record['ApproximateArrivalTimestamp'])
                 if not isinstance(record['Data'], str):
-                    record['Data'] = base64.encodebytes(bytearray(record['Data']['data']))
+                    record['Data'] = bytearray(record['Data']['data'])
 
             if encoding_type == APPLICATION_CBOR:
                 response._content = cbor2.dumps(results)
