@@ -2,9 +2,9 @@ import logging
 import traceback
 from localstack import config
 from localstack.utils.aws import aws_stack
-from localstack.utils.common import mkdir, get_free_tcp_port, edge_ports_info
+from localstack.utils.common import mkdir, get_free_tcp_port
 from localstack.services import install
-from localstack.services.infra import start_proxy_for_service, do_run
+from localstack.services.infra import start_proxy_for_service, do_run, log_startup_message
 from localstack.services.install import ROOT_PATH
 
 LOGGER = logging.getLogger(__name__)
@@ -27,7 +27,7 @@ def start_kinesis(port=None, asynchronous=False, update_listener=None):
         ROOT_PATH, config.KINESIS_SHARD_LIMIT, backend_port,
         latency, latency, latency, kinesis_data_dir_param
     )
-    print('Starting mock Kinesis service on %s ...' % edge_ports_info())
+    log_startup_message('Kinesis')
     start_proxy_for_service('kinesis', port, backend_port, update_listener)
     return do_run(cmd, asynchronous)
 
