@@ -12,7 +12,6 @@ import collections
 import dateutil.parser
 import urllib.parse
 import six
-import functools
 import botocore.config
 from pytz import timezone
 from urllib.parse import parse_qs
@@ -251,9 +250,9 @@ def send_notification_for_subscriber(notif, bucket_name, object_path, version_id
         sns_client = aws_stack.connect_to_service('sns')
         try:
             sns_client.publish(TopicArn=notif['Topic'], Message=message, Subject='Amazon S3 Notification')
-        except Exception:
-            LOGGER.warning('Unable to send notification for S3 bucket "%s" to SNS topic "%s".' %
-                (bucket_name, notif['Topic']))
+        except Exception as e:
+            LOGGER.warning('Unable to send notification for S3 bucket "%s" to SNS topic "%s": %s' %
+                (bucket_name, notif['Topic'], e))
     # CloudFunction and LambdaFunction are semantically identical
     lambda_function_config = notif.get('CloudFunction') or notif.get('LambdaFunction')
     if lambda_function_config:
@@ -272,13 +271,14 @@ def send_notification_for_subscriber(notif, bucket_name, object_path, version_id
             '/'.join(NOTIFICATION_DESTINATION_TYPES))
 
 
+# TODO: refactor/unify the 3 functions below...
 def get_cors(bucket_name):
     bucket_name = normalize_bucket_name(bucket_name)
     response = Response()
 
     exists, code = bucket_exists(bucket_name)
     if not exists:
-        response.status_code = code
+        response.status_code = int(code)
         return response
 
     cors = BUCKET_CORS.get(bucket_name)
@@ -298,7 +298,7 @@ def set_cors(bucket_name, cors):
 
     exists, code = bucket_exists(bucket_name)
     if not exists:
-        response.status_code = code
+        response.status_code = int(code)
         return response
 
     if not isinstance(cors, dict):
@@ -315,7 +315,7 @@ def delete_cors(bucket_name):
 
     exists, code = bucket_exists(bucket_name)
     if not exists:
-        response.status_code = code
+        response.status_code = int(code)
         return response
 
     BUCKET_CORS.pop(bucket_name, {})
@@ -435,7 +435,7 @@ def is_url_already_expired(expiry_timestamp):
     return False
 
 
-def add_reponse_metadata_headers(response):
+def add_response_metadata_headers(response):
     if response.headers.get('content-language') is None:
         response.headers['content-language'] = 'en-US'
     if response.headers.get('cache-control') is None:
@@ -552,7 +552,7 @@ def fix_metadata_key_underscores(request_headers={}, response=None):
             if key != key_new:
                 request_headers[key_new] = request_headers.pop(key)
                 updated = True
-    if response:
+    if response is not None:
         for key in list(response.headers.keys()):
             if key.lower().startswith(meta_header_prefix):
                 key_new = meta_header_prefix + key[prefix_len:].replace(underscore_replacement, '_')
@@ -579,57 +579,6 @@ def fix_delimiter(data, headers, response):
             response._content = re.compile(pattern).sub(delimiter, c)
 
 
-def fix_sorting_versions(method, parsed, response):
-
-    try:
-        if method != 'GET':
-            return
-
-        if not parsed or not parsed.query or 'versions' not in parsed.query:
-            return
-        try:
-            content = to_str(response._content)
-        except Exception:
-            # return in case of conversion errors
-            return
-
-        if not content or '<ListVersionsResult' not in content:
-            return
-
-        parsed = xmltodict.parse(content).get('ListVersionsResult')
-        if not isinstance(parsed.get('Version'), list):
-            return
-
-        result = list(parsed.get('Version'))
-
-        def compare(item1, item2):
-            # sort based on key in ascending order
-            if item1['Key'] < item2['Key']:
-                return -1
-            elif item1['Key'] > item2['Key']:
-                return 1
-            else:
-                # sort based on last modified in descending order
-                t1 = time.mktime(datetime.datetime.strptime(item1['LastModified'], '%Y-%m-%dT%H:%M:%S.%fZ').timetuple())
-                t2 = time.mktime(datetime.datetime.strptime(item2['LastModified'], '%Y-%m-%dT%H:%M:%S.%fZ').timetuple())
-                if t1 > t2:
-                    return -1
-                elif t1 < t2:
-                    return 1
-                else:
-                    # sort based on latest
-                    if item1['IsLatest'] == 'true':
-                        return -1
-                    elif item2['IsLatest'] == 'true':
-                        return 1
-            return 0
-
-        parsed['Version'] = sorted(result, key=functools.cmp_to_key(compare))
-        response._content = xmltodict.unparse({'ListVersionsResult': parsed})
-    except Exception:
-        return
-
-
 def convert_to_chunked_encoding(method, path, response):
     if method != 'GET' or path != '/':
         return
@@ -1379,14 +1328,13 @@ class ProxyListenerS3(PersistingProxyListener):
             ret304_on_etag(data, headers, response)
             append_aws_request_troubleshooting_headers(response)
             fix_delimiter(data, headers, response)
-            fix_sorting_versions(method, parsed, response)
 
             if method == 'PUT':
                 set_object_expiry(path, headers)
 
             # Remove body from PUT response on presigned URL
             # https://github.com/localstack/localstack/issues/1317
-            if method == 'PUT' and response.status_code < 400 and ('X-Amz-Security-Token=' in path or
+            if method == 'PUT' and int(response.status_code) < 400 and ('X-Amz-Security-Token=' in path or
                     'X-Amz-Credential=' in path or 'AWSAccessKeyId=' in path):
                 response._content = ''
                 reset_content_length = True
@@ -1401,7 +1349,7 @@ class ProxyListenerS3(PersistingProxyListener):
             # https://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectGET.html
             if method == 'GET':
                 add_accept_range_header(response)
-                add_reponse_metadata_headers(response)
+                add_response_metadata_headers(response)
                 if is_object_expired(path):
                     return no_such_key_error(path, headers.get('x-amz-request-id'), 400)
 
@@ -1440,7 +1388,7 @@ class ProxyListenerS3(PersistingProxyListener):
 
                 reset_content_length = True
 
-            # update content-length headers (fix https://github.com/localstack/localstack/issues/541)
+            # update Content-Length headers (fix https://github.com/localstack/localstack/issues/541)
             if method == 'DELETE':
                 reset_content_length = True
 
