@@ -17,7 +17,7 @@ from localstack.utils.analytics import event_publisher
 from localstack.utils.aws import aws_stack
 from localstack.utils.aws.aws_responses import response_regex_replace
 from localstack.utils.aws.dead_letter_queue import sns_error_to_dead_letter_queue
-from localstack.utils.common import timestamp_millis, short_uid, to_str, to_bytes, start_thread
+from localstack.utils.common import parse_request_data, timestamp_millis, short_uid, to_str, to_bytes, start_thread
 from localstack.utils.persistence import PersistingProxyListener
 
 # set up logger
@@ -38,6 +38,9 @@ PLATFORM_ENDPOINT_MESSAGES = {}
 # maps phone numbers to list of sent messages
 SMS_MESSAGES = []
 
+# actions to be skipped from persistence
+SKIP_PERSISTENCE_ACTIONS = ['Subscribe', 'ConfirmSubscription', 'Unsubscribe']
+
 
 class ProxyListenerSNS(PersistingProxyListener):
     def api_name(self):
@@ -242,6 +245,13 @@ class ProxyListenerSNS(PersistingProxyListener):
                     payload={'t': event_publisher.get_hash(topic_arn)}
                 )
 
+    def should_persist(self, method, path, data, headers, response):
+        req_params = parse_request_data(method, path, data)
+        action = req_params.get('Action', '')
+        if action in SKIP_PERSISTENCE_ACTIONS:
+            return False
+        return super(ProxyListenerSNS, self).should_persist(method, path, data, headers, response)
+
 
 # instantiate listener
 UPDATE_SNS = ProxyListenerSNS()
@@ -365,6 +375,16 @@ def message_to_subscribers(message_id, message, topic_arn, req_data, subscriptio
                 LOG.warning('Unable to forward SNS message to SNS platform app: %s %s' % (exc, traceback.format_exc()))
                 sns_error_to_dead_letter_queue(subscriber['SubscriptionArn'], req_data, str(exc))
 
+        elif subscriber['Protocol'] == 'email':
+            ses_client = aws_stack.connect_to_service('ses')
+            if subscriber.get('Endpoint'):
+                ses_client.verify_email_address(EmailAddress=subscriber.get('Endpoint'))
+                ses_client.verify_email_address(EmailAddress='admin@localstack.com')
+
+                ses_client.send_email(Source='admin@localstack.com',
+                                      Message={'Body': {'Text': {'Data': message}},
+                                               'Subject': {'Data': 'SNS-Subscriber-Endpoint'}},
+                                      Destination={'ToAddresses': [subscriber.get('Endpoint')]})
         else:
             LOG.warning('Unexpected protocol "%s" for SNS subscription' % subscriber['Protocol'])
 
