@@ -1,16 +1,15 @@
 import re
 import json
 import binascii
+import datetime
 import xmltodict
 from flask import Response
 from binascii import crc32
 from requests.models import CaseInsensitiveDict
 from requests.models import Response as RequestsResponse
-from localstack.utils.common import to_str, to_bytes
 from localstack.constants import TEST_AWS_ACCOUNT_ID, MOTO_ACCOUNT_ID
 from localstack.utils.aws import aws_stack
-from localstack.utils.common import short_uid
-import datetime
+from localstack.utils.common import short_uid, to_str, to_bytes, json_safe, replace_response_content
 
 REGEX_FLAGS = re.MULTILINE | re.DOTALL
 
@@ -32,8 +31,8 @@ def flask_error_response_json(msg, code=500, error_type='InternalFailure'):
     return Response(json.dumps(result), status=code, headers=headers)
 
 
-def requests_error_response_json(msg, code=500, error_type='InternalFailure'):
-    response = flask_error_response_json(msg, code=code, error_type=error_type)
+def requests_error_response_json(message, code=500, error_type='InternalFailure'):
+    response = flask_error_response_json(message, code=code, error_type=error_type)
     return flask_to_requests_response(response)
 
 
@@ -50,6 +49,23 @@ def requests_error_response_xml(message, code=400, code_string='InvalidParameter
     return response
 
 
+def requests_response_xml(action, response, xmlns=None, service=None):
+    xmlns = xmlns or 'http://%s.amazonaws.com/doc/2010-03-31/' % service
+    response = json_safe(response)
+    response = {'{action}Result'.format(action=action): response}
+    response = xmltodict.unparse(response)
+    if response.startswith('<?xml'):
+        response = re.sub(r'<\?xml [^\?]+\?>', '', response)
+    result = ("""
+        <{action}Response xmlns="{xmlns}">
+            {response}
+        </{action}Response>
+    """).strip()
+    result = result.format(action=action, xmlns=xmlns, response=response)
+    result = requests_response(result)
+    return result
+
+
 def requests_error_response_xml_signature_calculation(message, string_to_sign=None, signature=None, expires=None,
         code=400, code_string='AccessDenied', aws_access_token='temp'):
     response = RequestsResponse()
@@ -104,7 +120,7 @@ def requests_error_response(req_headers, message, code=500, error_type='Internal
     accept = req_headers.get('Accept', '')
     is_json = 'json' in ctype or 'json' in accept
     if is_json:
-        return requests_error_response_json(msg=message, code=code, error_type=error_type)
+        return requests_error_response_json(message=message, code=code, error_type=error_type)
     return requests_error_response_xml(message, code=code, code_string=error_type, service=service, xmlns=xmlns)
 
 
@@ -112,8 +128,9 @@ def requests_response(content, status_code=200, headers={}):
     resp = RequestsResponse()
     content = json.dumps(content) if isinstance(content, dict) else content
     resp._content = content
-    resp.status_code = status_code
-    resp.headers = headers
+    resp.status_code = int(status_code)
+    # Note: update headers (instead of assigning directly), to ensure we're using a case-insensitive dict
+    resp.headers.update(headers or {})
     return resp
 
 
@@ -168,19 +185,14 @@ class MessageConversion(object):
     @staticmethod
     def fix_date_format(response):
         """ Normalize date to format '2019-06-13T18:10:09.1234Z' """
-
-        def _replace(response, pattern, replacement):
-            content = to_str(response.content)
-            response._content = re.sub(pattern, replacement, content)
-
         pattern = r'<CreateDate>([^<]+) ([^<+]+)(\+[^<]*)?</CreateDate>'
         replacement = r'<CreateDate>\1T\2Z</CreateDate>'
-        _replace(response, pattern, replacement)
+        replace_response_content(response, pattern, replacement)
 
     @staticmethod
     def fix_account_id(response):
         return aws_stack.fix_account_id_in_arns(
-            response, existing=MOTO_ACCOUNT_ID, replace=TEST_AWS_ACCOUNT_ID)
+            response, replace=TEST_AWS_ACCOUNT_ID)
 
     @staticmethod
     def fix_error_codes(method, data, response):
