@@ -294,11 +294,14 @@ def check_valid_region(headers):
         raise Exception('Invalid region specified in "Authorization" header: "%s"' % region)
 
 
-def set_default_region_in_headers(headers):
+def set_default_region_in_headers(headers, service=None, region=None):
     auth_header = headers.get('Authorization')
+    region = region or get_region()
     if not auth_header:
+        if service:
+            headers['Authorization'] = mock_aws_request_headers(service, region_name=region)['Authorization']
         return
-    replaced = re.sub(r'(.*Credential=[^/]+/[^/]+/)([^/])+/', r'\1%s/' % get_region(), auth_header)
+    replaced = re.sub(r'(.*Credential=[^/]+/[^/]+/)([^/])+/', r'\1%s/' % region, auth_header)
     headers['Authorization'] = replaced
 
 
@@ -317,7 +320,7 @@ def fix_account_id_in_arns(response, colon_delimiter=':', existing=None, replace
 
     if not is_str_obj:
         response._content = content
-        response.headers['content-length'] = len(response._content)
+        response.headers['Content-Length'] = len(response._content)
         return response
     return content
 
@@ -353,6 +356,11 @@ def extract_region_from_arn(arn):
     return parts[3] if len(parts) > 1 else None
 
 
+def extract_service_from_arn(arn):
+    parts = arn.split(':')
+    return parts[2] if len(parts) > 1 else None
+
+
 def get_account_id(account_id=None, env=None):
     if account_id:
         return account_id
@@ -420,6 +428,11 @@ def dynamodb_stream_arn(table_name, latest_stream_label, account_id=None):
         (get_region(), account_id, table_name, latest_stream_label))
 
 
+def cloudwatch_alarm_arn(alarm_name, account_id=None, region_name=None):
+    pattern = 'arn:aws:cloudwatch:%s:%s:alarm:%s'
+    return _resource_arn(alarm_name, pattern, account_id=account_id, region_name=region_name)
+
+
 def log_group_arn(group_name, account_id=None, region_name=None):
     pattern = 'arn:aws:logs:%s:%s:log-group:%s'
     return _resource_arn(group_name, pattern, account_id=account_id, region_name=region_name)
@@ -524,7 +537,7 @@ def _resource_arn(name, pattern, account_id=None, region_name=None):
     return pattern % (region_name, account_id, name)
 
 
-def send_event_to_target(arn, event):
+def send_event_to_target(arn, event, target_attributes=None):
     if ':lambda:' in arn:
         from localstack.services.awslambda import lambda_api
         lambda_api.run_lambda(event=event, context={}, func_arn=arn)
@@ -536,7 +549,10 @@ def send_event_to_target(arn, event):
     elif ':sqs:' in arn:
         sqs_client = connect_to_service('sqs')
         queue_url = get_sqs_queue_url(arn)
-        sqs_client.send_message(QueueUrl=queue_url, MessageBody=json.dumps(event))
+
+        msg_group_id = (target_attributes or {}).get('MessageGroupId')
+        kwargs = {'MessageGroupId': msg_group_id} if msg_group_id else {}
+        sqs_client.send_message(QueueUrl=queue_url, MessageBody=json.dumps(event), **kwargs)
 
     elif ':states' in arn:
         stepfunctions_client = connect_to_service('stepfunctions')
@@ -546,6 +562,12 @@ def send_event_to_target(arn, event):
         LOG.info('Unsupported Events rule target ARN "%s"' % arn)
 
 
+def get_events_target_attributes(target):
+    # added for sqs, if needed can be moved to an if else
+    # block for multiple targets
+    return target.get('SqsParameters')
+
+
 def create_sqs_queue(queue_name, env=None):
     env = get_environment(env)
     # queue
@@ -914,3 +936,14 @@ def await_stack_status(stack_name, expected_statuses, retries=3, sleep=2):
 def await_stack_completion(stack_name, retries=3, sleep=2, statuses=None):
     statuses = statuses or ['CREATE_COMPLETE', 'UPDATE_COMPLETE']
     return await_stack_status(stack_name, statuses, retries=retries, sleep=sleep)
+
+
+# TODO: move to aws_responses.py?
+def extract_tags(req_data):
+    tags = []
+    req_tags = {k: v for k, v in req_data.items() if k.startswith('Tags.member.')}
+    for i in range(int(len(req_tags.keys()) / 2)):
+        key = req_tags['Tags.member.' + str(i + 1) + '.Key']
+        value = req_tags['Tags.member.' + str(i + 1) + '.Value']
+        tags.append({'Key': key, 'Value': value})
+    return tags
