@@ -140,7 +140,8 @@ def load_plugins(scope=None):
         return PLUGINS_LOADED[scope]
 
     t1 = now_utc()
-    log_level = logging.WARNING if scope == PLUGIN_SCOPE_COMMANDS else None
+    is_infra_process = os.environ.get(constants.LOCALSTACK_INFRA_PROCESS) in ['1', 'true'] or '--host' in sys.argv
+    log_level = logging.WARNING if scope == PLUGIN_SCOPE_COMMANDS and not is_infra_process else None
     setup_logging(log_level=log_level)
 
     loaded_files = []
@@ -253,11 +254,23 @@ def get_server_version():
 
 
 def setup_logging(log_level=None):
-    # determine and set log level
+    """ Determine and set log level """
+
     if PLUGINS_LOADED.get('_logging_'):
         return
     PLUGINS_LOADED['_logging_'] = True
-    log_level = log_level or (logging.DEBUG if is_debug() else logging.INFO)
+
+    # log level set by DEBUG env variable
+    log_level = log_level or (logging.DEBUG if config.DEBUG else logging.INFO)
+
+    # overriding the log level if LS_LOG has been set
+    if config.LS_LOG:
+        LS_LOG = str(config.LS_LOG).upper()
+        LS_LOG = 'WARNING' if LS_LOG == 'WARN' else LS_LOG
+        log_level = getattr(logging, LS_LOG)
+        logging.getLogger('').setLevel(log_level)
+        logging.getLogger('localstack').setLevel(log_level)
+
     logging.basicConfig(level=log_level, format=LOG_FORMAT, datefmt=LOG_DATE_FORMAT)
 
     # set up werkzeug logger
@@ -269,7 +282,7 @@ def setup_logging(log_level=None):
     root_handlers = logging.getLogger().handlers
     if len(root_handlers) > 0:
         root_handlers[0].addFilter(WerkzeugLogFilter())
-        if is_debug():
+        if config.DEBUG:
             format = '%(asctime)s:API: %(message)s'
             handler = logging.StreamHandler()
             handler.setLevel(logging.INFO)
@@ -301,6 +314,8 @@ def canonicalize_api_names(apis=None):
                 including "iam", "lambda", "dynamodb", "apigateway", "s3", "sns", and "logs"), and
         (3) removing duplicates from the list. """
 
+    # TODO: cache the result, as the code below is a relatively expensive operation!
+
     apis = apis or list(config.SERVICE_PORTS.keys())
 
     def contains(apis, api):
@@ -510,7 +525,7 @@ def start_infra_in_docker():
 
     mkdir(config.TMP_FOLDER)
     try:
-        run('chmod -R 777 "%s"' % config.TMP_FOLDER)
+        run('chmod -R 777 "%s"' % config.TMP_FOLDER, print_error=False)
     except Exception:
         pass
 
@@ -676,7 +691,3 @@ def mkdir(folder):
             # Ignore rare 'File exists' race conditions.
             if err.errno != 17:
                 raise
-
-
-def is_debug():
-    return os.environ.get('DEBUG', '').strip() not in ['', '0', 'false']
