@@ -19,8 +19,10 @@ from localstack.utils.common import (
 from localstack.utils.testutil import create_zip_file, delete_all_s3_objects
 from localstack.utils.cloudformation import template_preparer
 from localstack.services.awslambda.lambda_api import get_handler_file_from_name
-from localstack.services.cloudformation.service_models import (
-    GenericBaseModel, DependencyNotYetSatisfied, PLACEHOLDER_RESOURCE_NAME, PLACEHOLDER_AWS_NO_VALUE)
+from localstack.services.cloudformation.service_models import GenericBaseModel, DependencyNotYetSatisfied
+from localstack.services.cloudformation.deployment_utils import (
+    dump_json_params, select_parameters, param_defaults, remove_none_values, params_list_to_dict,
+    lambda_keys_to_lower, PLACEHOLDER_AWS_NO_VALUE, PLACEHOLDER_RESOURCE_NAME)
 
 ACTION_CREATE = 'create'
 ACTION_DELETE = 'delete'
@@ -32,7 +34,7 @@ LOG = logging.getLogger(__name__)
 # list of resource types that can be updated
 # TODO: make this a property of the model classes themselves
 UPDATEABLE_RESOURCES = [
-    'Lambda::Function', 'ApiGateway::Method', 'StepFunctions::StateMachine', 'IAM::Role'
+    'Lambda::Function', 'ApiGateway::Method', 'StepFunctions::StateMachine', 'IAM::Role', 'EC2::Instance'
 ]
 
 # list of static attribute references to be replaced in {'Fn::Sub': '...'} strings
@@ -67,10 +69,6 @@ def lambda_get_params():
     return lambda params, **kwargs: params
 
 
-def lambda_keys_to_lower(key=None):
-    return lambda params, **kwargs: common.keys_to_lower(params.get(key) if key else params)
-
-
 def rename_params(func, rename_map):
     def do_rename(params, **kwargs):
         values = func(params, **kwargs) if func else params
@@ -80,17 +78,6 @@ def rename_params(func, rename_map):
     return do_rename
 
 
-def params_list_to_dict(param_name, key_attr_name='Key', value_attr_name='Value'):
-    def do_replace(params, **kwargs):
-        result = {}
-        for entry in params.get(param_name, []):
-            key = entry[key_attr_name]
-            value = entry[value_attr_name]
-            result[key] = value
-        return result
-    return do_replace
-
-
 def params_dict_to_list(param_name, key_attr_name='Key', value_attr_name='Value', wrapper=None):
     def do_replace(params, **kwargs):
         result = []
@@ -161,44 +148,10 @@ def es_add_tags_params(params, **kwargs):
     return {'ARN': es_arn, 'TagList': tags}
 
 
-def select_parameters(*param_names):
-    return lambda params, **kwargs: dict([(k, v) for k, v in params.items() if k in param_names])
-
-
 def merge_parameters(func1, func2):
     return lambda params, **kwargs: common.merge_dicts(func1(params, **kwargs), func2(params, **kwargs))
 
 
-def dump_json_params(param_func=None, *param_names):
-    def replace(params, **kwargs):
-        result = param_func(params, **kwargs) if param_func else params
-        for name in param_names:
-            if isinstance(result.get(name), (dict, list)):
-                # Fix for https://github.com/localstack/localstack/issues/2022
-                # Convert any date instances to date strings, etc, Version: "2012-10-17"
-                param_value = common.json_safe(result[name])
-                result[name] = json.dumps(param_value)
-        return result
-    return replace
-
-
-def param_defaults(param_func, defaults):
-    def replace(params, **kwargs):
-        result = param_func(params, **kwargs)
-        for key, value in defaults.items():
-            if result.get(key) in ['', None]:
-                result[key] = value
-        return result
-    return replace
-
-
-def iam_create_policy_params(params, **kwargs):
-    result = {'PolicyName': params['PolicyName']}
-    policy_doc = remove_none_values(params['PolicyDocument'])
-    result['PolicyDocument'] = json.dumps(policy_doc)
-    return result
-
-
 def lambda_permission_params(params, **kwargs):
     result = select_parameters('FunctionName', 'Action', 'Principal')(params, **kwargs)
     result['StatementId'] = common.short_uid()
@@ -465,13 +418,6 @@ RESOURCE_TO_FUNCTION = {
             }
         }
     },
-    'IAM::Policy': {
-        'create': {
-            'function': 'create_policy',
-            'parameters': iam_create_policy_params
-        }
-        # InlinePolicy in cloudformation will be deleted on deleting Role
-    },
     'ApiGateway::RestApi': {
         'create': {
             'function': 'create_rest_api',
@@ -597,6 +543,82 @@ RESOURCE_TO_FUNCTION = {
                 'Attributes': sns_subscription_params
             }
         }
+    },
+    'KMS::Key': {
+        'create': {
+            'function': 'create_key',
+            'parameters': {
+                'Policy': 'KeyPolicy'
+            }
+        },
+        'delete': {
+            # TODO Key need to be deleted in KMS backend
+            'function': 'schedule_key_deletion',
+            'parameters': {
+                'KeyId': 'PhysicalResourceId'
+            }
+        }
+    },
+    'EC2::Instance': {
+        'create': {
+            'function': 'create_instances',
+            'parameters': {
+                'InstanceType': 'InstanceType',
+                'SecurityGroups': 'SecurityGroups',
+                'KeyName': 'KeyName',
+                'ImageId': 'ImageId'
+            },
+            'defaults': {
+                'MinCount': 1,
+                'MaxCount': 1
+            }
+        },
+        'update': {
+            'function': 'modify_instance_attribute',
+            'parameters': {
+                'InstanceType': 'InstanceType',
+                'SecurityGroups': 'SecurityGroups',
+                'KeyName': 'KeyName',
+                'ImageId': 'ImageId'
+            }
+        },
+        'delete': {
+            'function': 'terminate_instances',
+            'parameters': {
+                'InstanceIds': lambda params, **kw: [kw['resources'][kw['resource_id']]['PhysicalResourceId']]
+            }
+        }
+    },
+    'EC2::SecurityGroup': {
+        'create': {
+            'function': 'create_security_group',
+            'parameters': {
+                'GroupName': 'GroupName',
+                'VpcId': 'VpcId',
+                'Description': 'GroupDescription'
+            }
+        },
+        'delete': {
+            'function': 'delete_security_group',
+            'parameters': {
+                'GroupId': 'PhysicalResourceId'
+            }
+        }
+    },
+    'IAM::InstanceProfile': {
+        'create': {
+            'function': 'create_instance_profile',
+            'parameters': {
+                'InstanceProfileName': 'InstanceProfileName',
+                'Path': 'Path'
+            }
+        },
+        'delete': {
+            'function': 'delete_instance_profile',
+            'parameters': {
+                'InstanceProfileName': 'InstanceProfileName'
+            }
+        }
     }
 }
 
@@ -681,7 +703,7 @@ def get_resource_name(resource):
         name = instance.get_resource_name()
 
     if not name:
-        LOG.info('Unable to extract name for resource type "%s"' % res_type)
+        LOG.debug('Unable to extract name for resource type "%s"' % res_type)
     return name
 
 
@@ -720,13 +742,13 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
     if resource_props is None:
         raise Exception('Unable to find properties for resource "%s": %s %s' % (resource_id, resource, resources))
     try:
-
         # try to look up resource class
         canonical_type = canonical_resource_type(resource_type)
         resource_class = RESOURCE_MODELS.get(canonical_type)
         if resource_class:
             instance = resource_class(resource)
             state = instance.fetch_state(stack_name=stack_name, resources=resources)
+            instance.update_state(state)
             return state
 
         # special case for stack parameters
@@ -753,7 +775,8 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
 
 def check_not_found_exception(e, resource_type, resource, resource_status):
     # we expect this to be a "not found" exception
-    markers = ['NoSuchBucket', 'ResourceNotFound', 'NoSuchEntity', 'NotFoundException', '404', 'not found', 'not exist']
+    markers = ['NoSuchBucket', 'ResourceNotFound', 'NoSuchEntity', 'NotFoundException',
+        '404', 'not found', 'not exist']
     if not list(filter(lambda marker, e=e: marker in str(e), markers)):
         LOG.warning('Unexpected error retrieving details for resource %s: %s %s - %s %s' %
             (resource_type, e, ''.join(traceback.format_stack()), resource, resource_status))
@@ -881,8 +904,8 @@ def canonical_resource_type(resource_type):
 
 def get_attr_from_model_instance(resource, attribute, resource_type, resource_id=None):
     resource_type = canonical_resource_type(resource_type)
-    # TODO: look up class from RESOURCE_MODELS instead of moto.MODEL_MAP here!
-    model_class = parsing.MODEL_MAP.get(resource_type)
+    # TODO: remove moto.MODEL_MAP here
+    model_class = RESOURCE_MODELS.get(resource_type) or parsing.MODEL_MAP.get(resource_type)
     if not model_class:
         if resource_type not in ['AWS::Parameter', 'Parameter']:
             LOG.debug('Unable to find model class for resource type "%s"' % resource_type)
@@ -934,7 +957,7 @@ def resolve_ref(stack_name, ref, resources, attribute):
     resource_type = get_resource_type(resource)
     result = extract_resource_attribute(resource_type, resource_new, attribute,
         resource_id=ref, resource=resource, resources=resources, stack_name=stack_name)
-    if not result:
+    if result is None:
         LOG.warning('Unable to extract reference attribute "%s" from resource: %s %s' %
             (attribute, resource_new, resource))
     return result
@@ -992,8 +1015,11 @@ def resolve_refs_recursively(stack_name, value, resources):
             if not result:
                 raise Exception('Cannot resolve fn::FindInMap: %s %s' % (value[keys_list[0]], list(resources.keys())))
 
-            result = result.get(value[keys_list[0]][2])
-            return result
+            key = value[keys_list[0]][2]
+            if not isinstance(key, str):
+                key = resolve_refs_recursively(stack_name, key, resources)
+
+            return result.get(key)
 
         if stripped_fn_lower == 'importvalue':
             import_value_key = resolve_refs_recursively(stack_name, value[keys_list[0]], resources)
@@ -1032,6 +1058,14 @@ def resolve_refs_recursively(stack_name, value, resources):
             string = resolve_refs_recursively(stack_name, string, resources)
             return string.split(delimiter)
 
+        if stripped_fn_lower == 'getazs':
+            region = resolve_refs_recursively(stack_name, value['Fn::GetAZs'], resources) or aws_stack.get_region()
+            azs = []
+            for az in ('a', 'b', 'c', 'd'):
+                azs.append('%s%s' % (region, az))
+
+            return azs
+
         if stripped_fn_lower == 'base64':
             value_to_encode = value[keys_list[0]]
             value_to_encode = resolve_refs_recursively(stack_name, value_to_encode, resources)
@@ -1153,21 +1187,6 @@ def convert_data_types(func_details, params):
     return result
 
 
-def remove_none_values(params):
-    """ Remove None values recursively in the given object. """
-    def remove_nones(o, **kwargs):
-        if isinstance(o, dict):
-            for k, v in dict(o).items():
-                if v is None:
-                    o.pop(k)
-        if isinstance(o, list):
-            common.run_safe(o.remove, None)
-            common.run_safe(o.remove, PLACEHOLDER_AWS_NO_VALUE)
-        return o
-    result = common.recurse_object(params, remove_nones)
-    return result
-
-
 # TODO remove this method
 def prepare_template_body(req_data):
     return template_preparer.prepare_template_body(req_data)
@@ -1191,6 +1210,18 @@ def delete_resource(resource_id, resources, stack_name):
             rs = iam_client.list_role_policies(RoleName=role_name)
             for policy in rs['PolicyNames']:
                 iam_client.delete_role_policy(RoleName=role_name, PolicyName=policy)
+
+            rs = iam_client.list_instance_profiles_for_role(RoleName=role_name)
+            for instance_profile in rs['InstanceProfiles']:
+                ip_name = instance_profile['InstanceProfileName']
+                iam_client.remove_role_from_instance_profile(
+                    InstanceProfileName=ip_name,
+                    RoleName=role_name
+                )
+                # iam_client.delete_instance_profile(
+                #     InstanceProfileName=ip_name
+                # )
+
         except Exception as e:
             if 'NoSuchEntity' not in str(e):
                 raise
@@ -1254,9 +1285,20 @@ def fix_resource_props_for_sdk_deployment(resource_type, resource_props):
         if 'ReceiveMessageWaitTimeSeconds' in resource_props:
             resource_props['ReceiveMessageWaitTimeSeconds'] = int(resource_props['ReceiveMessageWaitTimeSeconds'])
 
+    if resource_type == 'KMS::Key':
+        resource_props['KeyPolicy'] = json.dumps(resource_props.get('KeyPolicy', {}))
+        resource_props['Enabled'] = resource_props.get('Enabled', True)
+        resource_props['EnableKeyRotation'] = resource_props.get('EnableKeyRotation', False)
+        resource_props['Description'] = resource_props.get('Description', '')
+
 
 def configure_resource_via_sdk(resource_id, resources, resource_type, func_details, stack_name, action_name):
     resource = resources[resource_id]
+
+    if resource_type == 'EC2::Instance':
+        if action_name == 'create':
+            func_details['boto_client'] = 'resource'
+
     client = get_client(resource, func_details)
     function = getattr(client, func_details['function'])
     params = func_details.get('parameters') or lambda_get_params()
@@ -1469,6 +1511,14 @@ def run_post_create_actions(action_name, resource_id, resources, resource_type,
         for user in users:
             iam.attach_user_policy(UserName=user, PolicyArn=policy_arn)
 
+    elif resource_type == 'IAM::InstanceProfile':
+        if resource_props.get('Roles', []):
+            iam = aws_stack.connect_to_service('iam')
+            iam.add_role_to_instance_profile(
+                InstanceProfileName=resource_props['InstanceProfileName'],
+                RoleName=resource_props['Roles'][0]
+            )
+
 
 def is_none_or_empty_value(value):
     return not value or value == PLACEHOLDER_AWS_NO_VALUE
@@ -1539,18 +1589,34 @@ def determine_resource_physical_id(resource_id, resources=None, stack=None, attr
     if result is not None:
         # note that value could be an empty string here (in case of Parameter values)
         return result
+
     LOG.info('Unable to determine PhysicalResourceId for "%s" resource, ID "%s"' % (resource_type, resource_id))
 
 
-def update_resource_details(stack, resource_id, details):
+def update_resource_details(stack, resource_id, details, action=None):
     resource = stack.resources.get(resource_id, {})
     if not resource:
         return
+
     resource_type = resource.get('Type') or ''
     resource_type = re.sub('^AWS::', '', resource_type)
     resource_props = resource.get('Properties', {})
     if resource_type == 'ApiGateway::RestApi':
         resource_props['id'] = details['id']
+
+    if resource_type == 'KMS::Key':
+        resource['PhysicalResourceId'] = details['KeyMetadata']['KeyId']
+
+    if resource_type == 'EC2::Instance':
+        if action == 'CREATE':
+            stack.resources[resource_id]['PhysicalResourceId'] = details[0].id
+
+    if resource_type == 'EC2::SecurityGroup':
+        stack.resources[resource_id]['PhysicalResourceId'] = details['GroupId']
+
+    if resource_type == 'IAM::InstanceProfile':
+        stack.resources[resource_id]['PhysicalResourceId'] = details['InstanceProfile']['InstanceProfileName']
+
     if isinstance(details, MotoCloudFormationModel):
         # fallback: keep track of moto resource status
         stack.moto_resource_statuses[resource_id] = details
@@ -1567,6 +1633,8 @@ def add_default_resource_props(resource, stack_name, resource_name=None,
     def _generate_res_name():
         return '%s-%s-%s' % (stack_name, resource_name or resource_id, short_uid())
 
+    # TODO: move logic below into resource classes!
+
     if res_type == 'AWS::Lambda::EventSourceMapping' and not props.get('StartingPosition'):
         props['StartingPosition'] = 'LATEST'
 
@@ -1585,9 +1653,21 @@ def add_default_resource_props(resource, stack_name, resource_name=None,
     elif res_type == 'AWS::SQS::QueuePolicy' and not resource.get('PhysicalResourceId'):
         resource['PhysicalResourceId'] = _generate_res_name()
 
+    elif res_type == 'AWS::IAM::ManagedPolicy' and not resource.get('ManagedPolicyName'):
+        resource['ManagedPolicyName'] = _generate_res_name()
+
     elif res_type == 'AWS::ApiGateway::RestApi' and not props.get('Name'):
         props['Name'] = _generate_res_name()
 
+    elif res_type == 'AWS::ApiGateway::Stage' and not props.get('StageName'):
+        props['StageName'] = 'default'
+
+    elif res_type == 'AWS::ApiGateway::ApiKey' and not props.get('Name'):
+        props['Name'] = _generate_res_name()
+
+    elif res_type == 'AWS::ApiGateway::UsagePlan' and not props.get('UsagePlanName'):
+        props['UsagePlanName'] = _generate_res_name()
+
     elif res_type == 'AWS::DynamoDB::Table':
         update_dynamodb_index_resource(resource)
         props['TableName'] = props.get('TableName') or _generate_res_name()
@@ -1603,6 +1683,12 @@ def add_default_resource_props(resource, stack_name, resource_name=None,
     elif res_type == 'AWS::CloudFormation::Stack' and not props.get('StackName'):
         props['StackName'] = _generate_res_name()
 
+    elif res_type == 'AWS::EC2::SecurityGroup':
+        props['GroupName'] = props.get('GroupName') or _generate_res_name()
+
+    elif res_type == 'AWS::IAM::InstanceProfile':
+        props['InstanceProfileName'] = props.get('InstanceProfileName') or _generate_res_name()
+
     # generate default names for certain resource types
     default_attrs = (('AWS::IAM::Role', 'RoleName'), ('AWS::Events::Rule', 'Name'))
     for entry in default_attrs:
@@ -1767,15 +1853,19 @@ class TemplateDeployer(object):
     def update_resource_details(self, resource_id, result, stack=None, action='CREATE'):
         stack = stack or self.stack
         # update resource state
-        update_resource_details(stack, resource_id, result)
+        update_resource_details(stack, resource_id, result, action)
         # update physical resource id
         resource = stack.resources[resource_id]
+
         physical_id = resource.get('PhysicalResourceId')
+
         physical_id = physical_id or determine_resource_physical_id(resource_id, stack=stack)
         if not resource.get('PhysicalResourceId') or action == 'UPDATE':
             resource['PhysicalResourceId'] = physical_id
+
         # set resource status
         stack.set_resource_status(resource_id, '%s_COMPLETE' % action, physical_res_id=physical_id)
+
         return physical_id
 
     def get_change_config(self, action, resource, change_set_id=None):
@@ -1817,9 +1907,29 @@ class TemplateDeployer(object):
         old_res_props = old_resource['Properties'] = old_resource.get('Properties', {})
         for key, value in new_resource['Properties'].items():
             old_res_props[key] = value
+
         # overwrite original template entirely
         old_stack.template_original['Resources'][resource_id] = new_stack.template_original['Resources'][resource_id]
 
+    def apply_parameter_changes(self, old_stack, new_stack):
+        parameters = {p['ParameterKey']: p['ParameterValue'] for p in old_stack.metadata['Parameters']}
+
+        for key, value in new_stack.template['Parameters'].items():
+            parameters[key] = value.get('Default', parameters.get(key))
+
+        parameters.update({p['ParameterKey']: p['ParameterValue'] for p in new_stack.metadata['Parameters']})
+
+        for change_set in new_stack.change_sets:
+            parameters.update({p['ParameterKey']: p for p in change_set.metadata['Parameters']})
+
+        old_stack.metadata['Parameters'] = [
+            {
+                'ParameterKey': k,
+                'ParameterValue': v
+            }
+            for k, v in parameters.items() if v
+        ]
+
     def apply_changes(self, old_stack, new_stack, stack_name, change_set_id=None, initialize=False):
         old_resources = old_stack.template['Resources']
         new_resources = new_stack.template['Resources']
@@ -1828,6 +1938,8 @@ class TemplateDeployer(object):
         adds = [val for key, val in new_resources.items() if initialize or key not in old_resources]
         modifies = [val for key, val in new_resources.items() if key in old_resources]
 
+        self.apply_parameter_changes(old_stack, new_stack)
+
         # construct changes
         changes = []
         contains_changes = False
@@ -1936,4 +2048,5 @@ class TemplateDeployer(object):
         # update resource status and physical resource id
         stack_action = {'Add': 'CREATE', 'Remove': 'DELETE', 'Modify': 'UPDATE'}.get(action)
         self.update_resource_details(resource_id, result, stack=old_stack, action=stack_action)
+
         return result
