@@ -176,24 +176,15 @@ class ShellCommandThread(FuncThread):
         return not psutil.pid_exists(self.process.pid)
 
     def stop(self, quiet=False):
-        # Note: Do NOT import "psutil" at the root scope, as this leads
-        # to problems when importing this file from our test Lambdas in Docker
-        # (Error: libc.musl-x86_64.so.1: cannot open shared object file)
-        import psutil
-
         if getattr(self, 'stopped', False):
             return
-
         if not self.process:
             LOG.warning("No process found for command '%s'" % self.cmd)
             return
 
         parent_pid = self.process.pid
         try:
-            parent = psutil.Process(parent_pid)
-            for child in parent.children(recursive=True):
-                child.kill()
-            parent.kill()
+            kill_process_tree(parent_pid)
             self.process = None
         except Exception:
             if not quiet:
@@ -784,7 +775,7 @@ def download(url, path, verify_ssl=True):
     try:
         if not os.path.exists(os.path.dirname(path)):
             os.makedirs(os.path.dirname(path))
-        LOG.debug('Starting download from %s to %s (%s bytes)' % (url, path, r.headers.get('content-length')))
+        LOG.debug('Starting download from %s to %s (%s bytes)' % (url, path, r.headers.get('Content-Length')))
         with open(path, 'wb') as f:
             iter_length = 0
             iter_limit = 1000000  # print a log line for every 1MB chunk
@@ -985,6 +976,11 @@ def str_remove(string, index, end_index=None):
     return '%s%s' % (string[:index], string[end_index:])
 
 
+def is_sub_dict(child_dict, parent_dict):
+    """ Returns whether the first dict is a sub-dict (subset) of the second dict. """
+    return all(parent_dict.get(key) == val for key, val in child_dict.items())
+
+
 def cleanup(files=True, env=ENV_DEV, quiet=True):
     if files:
         cleanup_tmp_files()
@@ -1007,7 +1003,8 @@ def cleanup_threads_and_processes(quiet=True, debug=False):
     for proc in TMP_PROCESSES:
         try:
             print_debug('[shutdown] Cleaning up process: %s' % proc, debug)
-            proc.terminate()
+            kill_process_tree(proc.pid)
+            # proc.terminate()
         except Exception as e:
             print(e)
     # clean up async tasks
@@ -1027,6 +1024,19 @@ def cleanup_threads_and_processes(quiet=True, debug=False):
     clear_list(TMP_PROCESSES)
 
 
+def kill_process_tree(parent_pid):
+    # Note: Do NOT import "psutil" at the root scope
+    import psutil
+    parent_pid = getattr(parent_pid, 'pid', None) or parent_pid
+    parent = psutil.Process(parent_pid)
+    for child in parent.children(recursive=True):
+        try:
+            child.kill()
+        except Exception:
+            pass
+    parent.kill()
+
+
 def clear_list(list_obj):
     while len(list_obj):
         del list_obj[0]
@@ -1074,14 +1084,27 @@ def unzip(path, target_dir, overwrite=True):
         # Running the native command can be an order of magnitude faster in Alpine on Travis-CI
         flags = '-o' if overwrite else ''
         flags += ' -q'
-        return run('cd %s; unzip %s %s' % (target_dir, flags, path))
+        try:
+            return run('cd %s; unzip %s %s' % (target_dir, flags, path), print_error=False)
+        except Exception as e:
+            error_str = truncate(str(e))
+            LOG.info('Unable to use native "unzip" command (using fallback mechanism): %s' % error_str)
+
     try:
         zip_ref = zipfile.ZipFile(path, 'r')
     except Exception as e:
         LOG.warning('Unable to open zip file: %s: %s' % (path, e))
         raise e
-    # Make sure to preserve file permissions in the zip file
-    # https://www.burgundywall.com/post/preserving-file-perms-with-python-zipfile-module
+
+    def _unzip_file_entry(zip_ref, file_entry, target_dir):
+        """ Extracts a Zipfile entry and preserves permissions """
+        zip_ref.extract(file_entry.filename, path=target_dir)
+        out_path = os.path.join(target_dir, file_entry.filename)
+        perm = file_entry.external_attr >> 16
+        # Make sure to preserve file permissions in the zip file
+        # https://www.burgundywall.com/post/preserving-file-perms-with-python-zipfile-module
+        os.chmod(out_path, perm or 0o777)
+
     try:
         for file_entry in zip_ref.infolist():
             _unzip_file_entry(zip_ref, file_entry, target_dir)
@@ -1089,16 +1112,6 @@ def unzip(path, target_dir, overwrite=True):
         zip_ref.close()
 
 
-def _unzip_file_entry(zip_ref, file_entry, target_dir):
-    """
-    Extracts a Zipfile entry and preserves permissions
-    """
-    zip_ref.extract(file_entry.filename, path=target_dir)
-    out_path = os.path.join(target_dir, file_entry.filename)
-    perm = file_entry.external_attr >> 16
-    os.chmod(out_path, perm or 0o777)
-
-
 def untar(path, target_dir):
     mode = 'r:gz' if path.endswith('gz') else 'r'
     with tarfile.open(path, mode) as tar:
@@ -1437,8 +1450,9 @@ def isoformat_milliseconds(t):
         return t.isoformat()[:-3]
 
 
-def _replace(response, pattern, replacement):
-    content = to_str(response.content)
+# TODO move to aws_responses.py?
+def replace_response_content(response, pattern, replacement):
+    content = to_str(response.content or '')
     response._content = re.sub(pattern, replacement, content)
 
 
